==> de.huxhorn.lilith.data.converter-0.9.44-sources.jar: de/huxhorn/lilith/data/converter/ConverterRegistry.java <==
/*
 * Lilith - a log event viewer.
 * Copyright (C) 2007-2014 Joern Huxhorn
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Copyright 2007-2014 Joern Huxhorn
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.huxhorn.lilith.data.converter;

import java.util.HashMap;
import java.util.Map;

public class ConverterRegistry<T>
{
	private final Map<Class<?>, Converter<T>> converterMap = new HashMap<Class<?>, Converter<T>>();
	
	public void addConverter(Converter<T> converter)
	{
		if(converter == null)
		{
			throw new IllegalArgumentException("Converter must not be null!");
		}
		Class clazz = converter.getSourceClass();
		if(clazz == null)
		{
			throw new IllegalStateException("Converter "+converter+" did not return a source class!");
		}
		
		synchronized(converterMap)
		{
			converterMap.put(clazz, converter);
		}
	}
	
	/**
	 * Returns a converter for the given object/class or null if no such converter
	 * could be resolved.
	 *
	 * @param object the object for which a converter should be resolved.
	 * @return a converter for the given object.
	 */
	public Converter<T> resolveConverter(Object object)
	{
		if(object == null)
		{
			return null;
		}
		Class<?> clazz;
		if(object instanceof Class)
		{
			clazz = (Class<?>) object;
		}
		else
		{
			clazz = object.getClass();
		}
		synchronized(converterMap)
		{
			Converter<T> converter = converterMap.get(clazz);
			if(converter != null)
			{
				return converter;
			}
			// check inheritance
			for(Map.Entry<Class<?>, Converter<T>> current : converterMap.entrySet())
			{
				Class<?> key = current.getKey();
				if(key.isAssignableFrom(clazz))
				{
					return current.getValue();
				}
			}
			return null;
		}
	}
}

==> de.huxhorn.lilith.data.converter-8.3.0-sources.jar: de/huxhorn/lilith/data/converter/ConverterRegistry.java <==
/*
 * Lilith - a log event viewer.
 * Copyright (C) 2007-2014 Joern Huxhorn
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Copyright 2007-2014 Joern Huxhorn
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.huxhorn.lilith.data.converter;

import java.util.HashMap;
import java.util.Map;

public class ConverterRegistry<T>
{
	private final Map<Class<?>, Converter<T>> converterMap = new HashMap<>();

	public void addConverter(Converter<T> converter)
	{
		if(converter == null)
		{
			throw new IllegalArgumentException("Converter must not be null!");
		}
		Class clazz = converter.getSourceClass();
		if(clazz == null)
		{
			throw new IllegalStateException("Converter "+converter+" did not return a source class!");
		}

		synchronized(converterMap)
		{
			converterMap.put(clazz, converter);
		}
	}

	/**
	 * Returns a converter for the given object/class or null if no such converter
	 * could be resolved.
	 *
	 * @param object the object for which a converter should be resolved.
	 * @return a converter for the given object.
	 */
	public Converter<T> resolveConverter(Object object)
	{
		if(object == null)
		{
			return null;
		}
		Class<?> clazz;
		if(object instanceof Class)
		{
			clazz = (Class<?>) object;
		}
		else
		{
			clazz = object.getClass();
		}
		synchronized(converterMap)
		{
			Converter<T> converter = converterMap.get(clazz);
			if(converter != null)
			{
				return converter;
			}
			// check inheritance
			for(Map.Entry<Class<?>, Converter<T>> current : converterMap.entrySet())
			{
				Class<?> key = current.getKey();
				if(key.isAssignableFrom(clazz))
				{
					return current.getValue();
				}
			}
			return null;
		}
	}
}
